@@ -1,7 +1,7 @@
 /* signal.c: signal and miscellaneous routines for the ed line editor. */
 /*  GNU ed - The GNU line editor.
-    Copyright (C) 1993, 1994 Andrew Moore, Talke Studio
-    Copyright (C) 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 1993, 1994, 2006, 2007, 2008, 2009, 2010
+    Free Software Foundation, Inc.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,10 +19,12 @@
 
 #include <ctype.h>
 #include <errno.h>
+#include <limits.h>
 #include <setjmp.h>
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
+#include <termios.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
 
@@ -30,48 +32,50 @@
 
 
 jmp_buf jmp_state;
-static int mutex = 0;			/* If set, signals stay pending */
-static int _window_lines = 22;		/* scroll length: ws_row - 2 */
-static int _window_columns = 72;
-static char sighup_pending = 0;
-static char sigint_pending = 0;
+static int mutex = 0;			/* If > 0, signals stay pending */
+static int window_lines_ = 22;		/* scroll length: ws_row - 2 */
+static int window_columns_ = 72;
+static bool sighup_pending = false;
+static bool sigint_pending = false;
 
 
-void sighup_handler( int signum )
+static void sighup_handler( int signum )
   {
-  signum = 0;			/* keep compiler happy */
-  if( mutex ) sighup_pending = 1;
+  signum = 0;				/* keep compiler happy */
+  if( mutex ) sighup_pending = true;
   else
     {
-    char hb[] = "ed.hup";
-    char *hup = 0;             /* hup filename */
-    char *s;
-    int n, m = 0;
-
-    sighup_pending = 0;
-    if( last_addr() && write_file( "ed.hup", "w", 1, last_addr(), 1 ) < 0 &&
-        ( s = getenv( "HOME" ) ) != 0 &&
-        ( !( n = strlen( s ) ) ||
-          ( int )( n + ( m = *( s + n - 1 ) != '/' ) + sizeof( hb ) ) < path_max( 0 ) ) &&
-        ( hup = ( char *) malloc( n + m + sizeof( hb ) ) ) != 0 )
+    const char hb[] = "ed.hup";
+    sighup_pending = false;
+    if( last_addr() && modified() &&
+        write_file( hb, "w", 1, last_addr() ) < 0 )
       {
-      memcpy( hup, s, n );
-      memcpy( hup + n, m ? "/" : "", 1 );
-      memcpy( hup + n + m, hb, sizeof( hb ) );
-      write_file( hup, "w", 1, last_addr(), 1 );
+      char * const s = getenv( "HOME" );
+      const int len = ( s ? strlen( s ) : 0 );
+      const int need_slash = ( ( !len || s[len-1] != '/' ) ? 1 : 0 );
+      char * const hup = ( ( len + need_slash + (int)sizeof hb < path_max( 0 ) ) ?
+                    (char *) malloc( len + need_slash + sizeof hb ) : 0 );
+      if( len && hup )		/* hup filename */
+        {
+        memcpy( hup, s, len );
+        if( need_slash ) hup[len] = '/';
+        memcpy( hup + len + need_slash, hb, sizeof hb );
+        if( write_file( hup, "w", 1, last_addr() ) >= 0 ) exit( 0 );
+        }
+      exit( 1 );		/* hup file write failed */
       }
-    exit( 2 );
+    exit( 0 );
     }
   }
 
 
-void sigint_handler( int signum )
+static void sigint_handler( int signum )
   {
-  if( mutex ) sigint_pending = 1;
+  if( mutex ) sigint_pending = true;
   else
     {
     sigset_t set;
-    sigint_pending = 0;
+    sigint_pending = false;
     sigemptyset( &set );
     sigaddset( &set, signum );
     sigprocmask( SIG_UNBLOCK, &set, 0 );
@@ -80,7 +84,7 @@ void sigint_handler( int signum )
   }
 
 
-void sigwinch_handler( int signum )
+static void sigwinch_handler( int signum )
   {
 #ifdef TIOCGWINSZ
   struct winsize ws;            /* window size structure */
@@ -88,15 +92,15 @@ void sigwinch_handler( int signum )
   if( ioctl( 0, TIOCGWINSZ, (char *) &ws ) >= 0 )
     {
     /* Sanity check values of environment vars */
-    if( ws.ws_row > 2 && ws.ws_row < 600 ) _window_lines = ws.ws_row - 2;
-    if( ws.ws_col > 8 && ws.ws_col < 1800 ) _window_columns = ws.ws_col - 8;
+    if( ws.ws_row > 2 && ws.ws_row < 600 ) window_lines_ = ws.ws_row - 2;
+    if( ws.ws_col > 8 && ws.ws_col < 1800 ) window_columns_ = ws.ws_col - 8;
     }
 #endif
   signum = 0;			/* keep compiler happy */
   }
 
 
-int set_signal( int signum, void (*handler )( int ) )
+static int set_signal( int signum, void (*handler)( int ) )
   {
   struct sigaction new_action;
 
@@ -137,41 +141,43 @@ void set_signals( void )
   }
 
 
-void set_window_lines( const int lines ) { _window_lines = lines; }
-int window_columns( void ) { return _window_columns; }
-int window_lines( void ) { return _window_lines; }
+void set_window_lines( const int lines ) { window_lines_ = lines; }
+int window_columns( void ) { return window_columns_; }
+int window_lines( void ) { return window_lines_; }
 
 
 /* convert a string to int with out_of_range detection */
-char parse_int( int *i, const char *str, const char **tail )
+bool parse_int( int * const i, const char * const str, const char ** const tail )
   {
-  char *tmp;
+  char * tmp;
+  long li;
+
   errno = 0;
-  *i = strtol( str, &tmp, 10 );
+  *i = li = strtol( str, &tmp, 10 );
   if( tail ) *tail = tmp;
   if( tmp == str )
     {
     set_error_msg( "Bad numerical result" );
     *i = 0;
-    return 0;
+    return false;
     }
-  if( errno == ERANGE )
+  if( errno == ERANGE || li > INT_MAX || li < INT_MIN )
     {
     set_error_msg( "Numerical result out of range" );
     *i = 0;
-    return 0;
+    return false;
     }
-  return 1;
+  return true;
   }
 
 
 /* assure at least a minimum size for buffer `buf' */
-char resize_buffer( char **buf, int *size, int min_size )
+bool resize_buffer( char ** const buf, int * const size, const int min_size )
   {
   if( *size < min_size )
     {
     const int new_size = ( min_size < 512 ? 512 : ( min_size / 512 ) * 1024 );
-    char *new_buf = 0;
+    void * new_buf = 0;
     disable_interrupts();
     if( *buf ) new_buf = realloc( *buf, new_size );
     else new_buf = malloc( new_size );
@@ -180,35 +186,79 @@ char resize_buffer( char **buf, int *size, int min_size )
       show_strerror( 0, errno );
       set_error_msg( "Memory exhausted" );
       enable_interrupts();
-      return 0;
+      return false;
       }
     *size = new_size;
-    *buf = new_buf;
+    *buf = (char *)new_buf;
     enable_interrupts();
     }
-  return 1;
+  return true;
   }
 
 
-/* scan command buffer for next non-space char */
-const char *skip_blanks( const char *s )
+/* assure at least a minimum size for buffer `buf' */
+bool resize_line_buffer( const line_t *** const buf, int * const size,
+                         const int min_size )
   {
-  while( isspace( (unsigned char)*s ) && *s != '\n' ) ++s;
-  return s;
+  if( *size < min_size )
+    {
+    const int new_size = ( min_size < 512 ? 512 : ( min_size / 512 ) * 1024 );
+    void * new_buf = 0;
+    disable_interrupts();
+    if( *buf ) new_buf = realloc( *buf, new_size );
+    else new_buf = malloc( new_size );
+    if( !new_buf )
+      {
+      show_strerror( 0, errno );
+      set_error_msg( "Memory exhausted" );
+      enable_interrupts();
+      return false;
+      }
+    *size = new_size;
+    *buf = (const line_t **)new_buf;
+    enable_interrupts();
+    }
+  return true;
   }
 
 
-/* return copy of escaped string of at most length PATH_MAX */
-const char *strip_escapes( const char *s )
+/* assure at least a minimum size for buffer `buf' */
+bool resize_undo_buffer( undo_t ** const buf, int * const size,
+                         const int min_size )
   {
-  static char *file = 0;
-  static int filesz = 0;
+  if( *size < min_size )
+    {
+    const int new_size = ( min_size < 512 ? 512 : ( min_size / 512 ) * 1024 );
+    void * new_buf = 0;
+    disable_interrupts();
+    if( *buf ) new_buf = realloc( *buf, new_size );
+    else new_buf = malloc( new_size );
+    if( !new_buf )
+      {
+      show_strerror( 0, errno );
+      set_error_msg( "Memory exhausted" );
+      enable_interrupts();
+      return false;
+      }
+    *size = new_size;
+    *buf = (undo_t *)new_buf;
+    enable_interrupts();
+    }
+  return true;
+  }
 
+
+/* return unescaped copy of escaped string */
+const char * strip_escapes( const char * p )
+  {
+  static char * buf = 0;
+  static int bufsz = 0;
+  const int len = strlen( p );
   int i = 0;
 
-  if( !resize_buffer( &file, &filesz, path_max( 0 ) + 1 ) ) return 0;
+  if( !resize_buffer( &buf, &bufsz, len + 1 ) ) return 0;
   /* assert: no trailing escape */
-  while( ( file[i++] = ( (*s == '\\' ) ? *++s : *s ) ) )
-    s++;
-  return file;
+  while( ( buf[i++] = ( (*p == '\\' ) ? *++p : *p ) ) )
+    ++p;
+  return buf;
   }
